#! /bin/sh
# \
exec wish${TCL} "$0" ${1+"$@"}

#
# Tu Combola Example
#
#
# Stuart Cassoff
#
# Fall 2022
# Version 1.0
#


package require Tcl 8.6


namespace eval tu::combola { namespace eval examples { namespace eval tu_combola_example {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu_combola_example
		title	{Combola Example}
		descr	{Using Combola Combobox Enhancements}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	1.0
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key not \"$key\"known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


### Procecures copied from other places

namespace eval tu {
namespace eval tcl {
proc laddAndSort {list value {sort {-dictionary}}} {
	set i -1
	set values [lsort {*}$sort -index 0 [lmap e [linsert $list end $value] { list $e [incr i] }]]
	return [list [lsearch -index 1 $values $i] [lmap e $values { lindex $e 0 }]]
}

namespace export laddAndSort
}; # namespace eval tcl

namespace eval tk {
proc setEntryText {w text} {
	$w delete 0 end
	if {$text ne ""} { $w insert end $text }
}
namespace export setEntryText
}; # namespace eval tk
}; # namespace eval tu


### Tu Combola Example

proc go {} {
	package require Tk
	package require tu::combola
	namespace import ::tu::combola::combola

	variable Status ""
	variable Combo  ""
	variable Key    ""

	set W . ; set w {}
	bind $W <Key-Escape> [list destroy $W]
	bind $W <Control-q>  [list destroy $W]

	wm title $W [pkgconfig get title]

	set ns [namespace current]:

	set cmds [lsort [namespace eval :: {info commands {[a-zA-Z0-9_]*}}]]

	grid [set ksbFrame [ttk::labelframe $w.fKsb -text "KeySelect:Both (Ksb)"          ]] -sticky ew -row 0 -column 0
	grid [set kbeFrame [ttk::labelframe $w.fKbe -text "KeySelect:Both,Editable (Ksbe)"]] -sticky ew -row 0 -column 1
	grid [set ksnFrame [ttk::labelframe $w.fKsn -text "KeySelect:None (Ksn)"          ]] -sticky ew -row 1 -column 0
	grid [set knrFrame [ttk::labelframe $w.fKnr -text "KeySelect:None,Reversed"       ]] -sticky ew -row 1 -column 1
	grid [set kslFrame [ttk::labelframe $w.fKsl -text "KeySelect:List"                ]] -sticky ew -row 2 -column 0
	grid [set kscFrame [ttk::labelframe $w.fKsc -text "KeySelect:Combo"               ]] -sticky ew -row 2 -column 1
	grid [set ttkFrame [ttk::labelframe $w.fTtk -text "Ttk Regular"                   ]] -sticky ew -row 3 -column 0

	grid [set ksbCombo [combola combobox $ksbFrame.cKsb -values $cmds                           ]] -sticky ew
	grid [set kbeCombo [combola combobox $kbeFrame.cKbe -values $cmds -edit $ns:comboCallback   ]] -sticky ew
	grid [set ksnCombo [combola combobox $ksnFrame.cKsn -values $cmds -keyselect none           ]] -sticky ew
	grid [set knrCombo [combola combobox $knrFrame.cKnr -values $cmds -keyselect none -upis down]] -sticky ew
	grid [set kslCombo [combola combobox $kslFrame.cKsl -values $cmds -keyselect listbox        ]] -sticky ew
	grid [set kscCombo [combola combobox $kscFrame.cKsc -values $cmds -keyselect combobox       ]] -sticky ew
	grid [set ttkCombo [ttk::combobox    $ttkFrame.cTtk -values $cmds                           ]] -sticky ew

	grid [set infoFrame [ttk::labelframe $w.fInfo -text "How To Use"]] -sticky ew -columnspan 2
	grid [set infoBook [ttk::notebook $infoFrame.nInfo]] -sticky ew -columnspan 2
	grid [set buttonsFrame [ttk::labelframe $w.fButtons -text "Send Events To"]] -sticky ew -columnspan 2
	grid [ttk::label $w.l0 -textvariable $ns:Status] -sticky ew -columnspan 2

	set combos [list ksb kbe ksn knr ksl ksc]

	set r 0 ; set c 0
	foreach combo $combos {
		grid [ttk::radiobutton $buttonsFrame.r$combo -text [[set ${combo}Frame] cget -text] -value [set ${combo}Combo] -variable $ns:Combo] -row $r -column $c -sticky w
		if {[incr c] == 2} { set c 0; incr r }
	}

	foreach e [list GoNext GoPrev GoHome GoEnd GoNextPage GoPrevPage OpenList BeginEdit] {
		set ev <<Combola$e>>
		grid [ttk::button $buttonsFrame.b$e -text $e -command "event generate \[set $ns:Combo\] $ev"] -row $r -column $c -sticky ew
		if {[incr c] == 2} { set c 0; incr r }
	}

	set e Key
	set ev <<Combola$e>>
	grid [ttk::button $buttonsFrame.b$e -text $e -command "event generate \[set $ns:Combo\] $ev -data \[set $ns:Key\]"] -row $r -column 0 -sticky ew
	grid [ttk::entry $buttonsFrame.e$e -textvariable $ns:Key -validate key -validatecommand [list $ns:ValidateOneCharEntry %W %d %s %S] -width 2] -row $r -column 1 -sticky w -padx {2 0}

	$infoBook add [ttk::label $infoBook.lKsb -text [join {
		{Combo: <Up>, <Down>, <PgUp>, <PgDn>, <Home>, <End>,}
		{<Control-Home> or <Control-End> to navigate.}
		{<Space>, <Alt-Space> or <Alt-Down> to open list.}
		{<Key> or <Shift-Key> to navigate by key.}
		{}
		{}
		{List: <Up>, <Down>, <PgUp>, <PgDn>, <Home>, <End>,}
		{<Control-Home> or <Control-End> to navigate.}
		{<Space>, <Alt-Space>, <Alt-Up> or <Esc> to close list.}
		{<Key> or <Shift-Key> to navigate by key.}
	} \n]] -sticky nw -text Ksb
	$infoBook add [ttk::label $infoBook.lKbe -text [join {
		{Combo: <Up>, <Down>, <PgUp>, <PgDn>, <Home>, <End>,}
		{<Control-Home> or <Control-End> to navigate.}
		{<Space>, <Alt-Space> or <Alt-Down> to open list.}
		{<Key> or <Shift-Key> to navigate by key.}
		{<Control-Return> to edit. <Control-Return> or <Esc> to stop editing.}
		{}
		{List: <Up>, <Down>, <PgUp>, <PgDn>, <Home>, <End>,}
		{<Control-Home> or <Control-End> to navigate.}
		{<Space>, <Alt-Space>, <Alt-Up> or <Esc> to close list.}
		{<Key> or <Shift-Key> to navigate by key.}
	} \n]] -sticky nw -text Ksbe
	$infoBook add [ttk::label $infoBook.lKsn -text [join {
		{Combo: <Up>, <Down>, <PgUp>, <PgDn>,}
		{<Control-Home> or <Control-End> to navigate.}
		{<Alt-Space> or <Alt-Down> to open list.}
		{}
		{}
		{}
		{List: <Up>, <Down>, <PgUp>, <PgDn>, <Home>, <End>,}
		{<Control-Home> or <Control-End> to navigate.}
		{<Alt-Space>, <Alt-Up> or <Esc> to close list.}
	} \n]] -sticky nw -text Ksn
	grid [ttk::label $infoFrame.lExit -text {<Esc> or <Control-q> to exit.}] -sticky w

	foreach ww [list $W $W] c [list 0 1] {
		grid columnconfigure $ww $c -weight 1 -uniform combos
	}

	foreach combo [linsert $combos end ttk] {
		grid columnconfigure [set ${combo}Frame] 0 -weight 1
	}

	foreach ww [list $infoFrame] c [list 0] {
		grid columnconfigure $ww $c -weight 1
	}

	foreach ww [list $buttonsFrame $buttonsFrame] c [list 0 1] {
		grid columnconfigure $ww $c -weight 1 -uniform buttons
	}

	set Combo [set [lindex $combos 0]Combo]

	foreach combo $combos {
		event generate [set ${combo}Combo] <<CombolaGoHome>>
	}
	$ttkCombo current 0

	focus $Combo
}

proc comboCallback {mode combo {comboi {}}} {
	variable Status
	if {$mode eq "begin"} {
		if {[$combo get] eq "apply"} {
			event generate $combo <<CombolaCancelEdit>> -when tail
			return
		}
		focus $combo
		grab set $combo
		set Status "Editing: <Control-Return> to accept, <Escape> to discard."
	} elseif {$mode eq "end"} {
		set Status ""
		set eres [dict get $comboi eres]
		if {$eres eq "cancel"} return
		grab release $combo
		if {$eres eq "discard"} return
		set value [string trim [dict get $comboi value]]
		set values [$combo cget -values]
		if {$value in $values} return
		lassign [laddAndSort $values $value] idx values
		$combo configure -values $values
		if 0 { # Two ways to the same result
			event generate $combo <<CombolaGo>> -data $idx
		} else {
			combola go $combo $idx
		}
	}
}

proc ValidateOneCharEntry {w d s S} {
        if {$d == 1 && $S ne ""} {
                setEntryText $w $S
        }
        return true
}

namespace import tu::tcl::laddAndSort
namespace import tu::tk::setEntryText

}}}; # namespace tu::combola::examples::tu_combola_example


if {[info exists ::argv0] && [info script] eq $::argv0} {
	tu::combola::examples::tu_combola_example::go {*}$::argv
}


# EOF
