#
# Tu Combola - Tcl Usefulness Combobox Enhancements
#
#
# Stuart Cassoff
#
# Summer 2022
# Version 0.1
#
# Fall 2022
# Version 0.2
#

package require Tcl 8.5

namespace eval tu { namespace eval combola {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu::combola
		title	{Tu Combola}
		descr	{Combobox Enhancements}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	0.2
	}]
	dict set pkgconfig importfrom [namespace parent]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


### Procedures copied from other places

namespace eval tu {
namespace eval tcl {
proc getNextOrPrevMatch {aList aString {curIdx 0} {backwards no}} {
	set ixs [lsearch -all -nocase $aList $aString*]
	if {[llength $ixs] == 0} { return -1 }
	if {[llength $ixs] == 1 && [lindex $ixs 0] == $curIdx} { return -1 }
	set p -1
	foreach ix $ixs {
		if {$ix < $curIdx} { set p $ix }
		if {$ix > $curIdx} break
	}
	if {$backwards} {
		if {$p == -1} {
			set idx [lindex $ixs end]
		} else {
			set idx $p
		}	
	} else {
		if {$ix == $curIdx} {
			set idx [lindex $ixs 0]
		} else {
			set idx $ix
		}
	}
	return $idx
}

proc whereBe {where curIdx nitems {height {10}}} {
	set idx $curIdx
	switch -exact -- $where {
	home     {  set idx  0       }
	end      {  set idx  $nitems }
	next     { incr idx  1       }
	prev     { incr idx -1       }
	nextpage { incr idx  $height ; if {$idx > $nitems} { set idx $nitems } }
	prevpage { incr idx -$height ; if {$idx < 0      } { set idx 0       } }
	default  {  set idx  $where  }
	}
	if {$idx < 0 || $idx > $nitems || $idx == $curIdx} { set idx -1 }
	return $idx
}

namespace export getNextOrPrevMatch whereBe
}; # namespace eval tcl

namespace eval tk {
proc bindAndBreak {w args} {
	dict for {seq script} $args {
		if {$script eq ""} {
			bind $w $seq break
		} elseif {$script in [list break continue]} {
			bind $w $seq $script
		} else {
			bind $w $seq $script
			bind $w $seq +break
		}
	}
}

namespace export bindAndBreak
}; # namespace eval tk
}; # namespace eval tu


### Combola

## Combo

proc comboGoto {combo idx} {
	$combo current $idx
	event generate $combo <<SelectAll>>
	event generate $combo <<ComboboxSelected>> -when tail
}

proc combolistGoto {combolist idx} {
	$combolist activate $idx
	$combolist see $idx
	event generate $combolist <<Invoke>> -when tail
}

proc comboGo {combo where} {
	set curIdx [$combo current]
	set nitems [expr {[llength [$combo cget -values]] - 1}]
	set height 10
	if {[set idx [whereBe $where $curIdx $nitems $height]] >= 0} {
		comboGoto $combo $idx
	}
}

proc combolistGo {combolist where} {
	set curIdx [CombolistCurrent $combolist]
	set nitems [expr {[$combolist index end] - 1}]
	set height [$combolist cget -height]
	if {[set idx [whereBe $where $curIdx $nitems $height]] >= 0} {
		combolistGoto $combolist $idx
	}
}

proc comboKey {combo key} {
	if {[string length $key] != 1} return
	set values [$combo cget -values]
	set curIdx [$combo current]
	set backwards [string is upper $key]
	if {[set idx [getNextOrPrevMatch $values $key $curIdx $backwards]] >= 0} {
		comboGoto $combo $idx
	}
}

proc combolistKey {combolist key} {
	if {[string length $key] != 1} return
	# XXX Very hacky for Ttk. Who knows if/when this may change?
	set values [[winfo parent [winfo parent [winfo parent $combolist]]] cget -values]
	set curIdx [CombolistCurrent $combolist]
	set backwards [string is upper $key]
	if {[set idx [getNextOrPrevMatch $values $key $curIdx $backwards]] >= 0} {
		combolistGoto $combolist $idx
	}
}

proc combolaBeginEdit {combo} {
	set curIdx [$combo current]
	set ns [namespace current]:
	set bt [bindtags $combo]
	set tag [MkTag $combo EditStop]
	set ix [lsearch -exact $bt CombolaKeySelectTCombobox]
	if {$ix == -1} { set ix [lsearch -exact $bt CombolaTCombobox] }
	set comboi [dict create lastIdx $curIdx combo $combo bindtags $bt tag $tag]
	bind $tag <<CombolaAcceptEdit>>  [list $ns:combolaEndEdit [dict replace $comboi eres accept]]
	bind $tag <<CombolaDiscardEdit>> [list $ns:combolaEndEdit [dict replace $comboi eres discard]]
	bind $tag <<CombolaCancelEdit>>  [list $ns:combolaEndEdit [dict replace $comboi eres cancel]]
	set bt [lreplace $bt $ix $ix $tag CombolaEditTCombobox]
	foreach seq [list . all] {
		set ix [lsearch -exact $bt $seq]
		set bt [lreplace $bt $ix $ix]
	}
	bindtags $combo $bt
	$combo state !readonly
	event generate $combo <<CombolaEditBegin>> -data [dict create value [$combo get]] -when tail
}

proc combolaEndEdit {comboi} {
	set combo [dict get $comboi combo]
	set value [$combo get]
	bindtags $combo [dict get $comboi bindtags]
	$combo state readonly
	set lastIdx [dict get $comboi lastIdx]
	if {$lastIdx >= 0} {
		comboGoto $combo $lastIdx
	} else {
		$combo set ""
	}
	RmTag [dict get $comboi tag]
	event generate $combo <<CombolaEditEnd>> -data [dict create value $value eres [dict get $comboi eres]] -when tail
}


## Commands

proc enhance {combo args} {
	Init
	set tags [list]
	set upis [expr {[dict exists $args -upis] ? [dict get $args -upis] : "up"}]
	if {$upis ni [list down up]} {
		return -code error "bad value \"$upis\": must be one of down, or up"
	}
	set keyselect [expr {[dict exists $args -keyselect] ? [dict get $args -keyselect] : "both"}]
	if {$keyselect ni [list both combobox listbox none]} {
		return -code error "bad value \"$keyselect\": must be one of both, combobox, listbox, or none"
	}
	if {$keyselect in [list combobox both]} {
		lappend tags CombolaKeySelectTCombobox
	}
	if {[dict exists $args -edit]} {
		set edit [dict get $args -edit]
		set tag [MkTag $combo EditReady]
		bind $tag <<CombolaEditBegin>> [list uplevel #0 [list $edit begin %W %d]]
		bind $tag <<CombolaEditEnd>>   [list uplevel #0 [list $edit end   %W %d]]
		bind $tag         <Destroy>    [list [namespace current]::RmTag $tag]
		lappend tags $tag CombolaEditReadyTCombobox
	}
	if {$keyselect in [list listbox both]} {
		lappend tags CombolaOpenKeySelectTCombobox
	} else {
		lappend tags CombolaOpenTCombobox
	}
	lappend tags CombolaScrollableUpIs[string totitle $upis]
	lappend tags CombolaTCombobox
	set idx [lsearch -exact [set bt [bindtags $combo]] [winfo class $combo]]
	set bt [linsert $bt $idx {*}$tags]
	bindtags $combo $bt
	if {$keyselect in [list combobox both]} {
		$combo state readonly
	}
	return $combo
}

proc combobox {w args} {
	Init
	lassign [Args $args] targs cargs
	return [enhance [ttk::combobox $w {*}$targs] {*}$cargs]
}


## Util

proc Args {argies} {
	set targs [dict create]
	set cargs [dict create]
	dict for {arg val} $argies {
		dict set [expr {$arg in [list -edit -keyselect -upis] ? "c" : "t"}]args $arg $val
	}
	return [list $targs $cargs]
}

proc CombolistCurrent {combolist} {
	set sel [$combolist curselection]
	return [expr {$sel ne {} ? [lindex $sel 0] : -1}]
}

proc MkEgt {event args} { list event generate %W $event {*}$args -when tail }
proc MkTag {combo sufx} { format {Combola`%s`%s} $combo $sufx }
proc RmTag {tag}        { foreach seq [bind $tag] { bind $tag $seq {} } }


## Init

proc Init {} {
	variable Initted;
	if {$Initted} return

	set ns [namespace current]:

	# Combobox
	bind CombolaTCombobox <<CombolaKey>> [list $ns:comboKey %W %d]
	bind CombolaTCombobox <<CombolaGo>>  [list $ns:comboGo  %W %d]

	# Listbox
	bind CombolaTComboboxListbox <<CombolaListKey>> [list $ns:combolistKey %W %d]
	bind CombolaTComboboxListbox <<CombolaListGo>>  [list $ns:combolistGo  %W %d]

	# Edit Ready
	bind CombolaEditReadyTCombobox <<CombolaBeginEdit>> [list $ns:combolaBeginEdit %W]

	# Open
	bind CombolaOpenTCombobox          <<CombolaOpenList>> [list ::ttk::combobox::CombolaPost %W]
	bind CombolaOpenKeySelectTCombobox <<CombolaOpenList>> [list ::ttk::combobox::CombolaKeySelectPost %W]

	# Combobox
	bind CombolaTCombobox <<CombolaGoNext>>     [MkEgt <<CombolaGo>> -data next    ]
	bind CombolaTCombobox <<CombolaGoPrev>>     [MkEgt <<CombolaGo>> -data prev    ]
	bind CombolaTCombobox <<CombolaGoHome>>     [MkEgt <<CombolaGo>> -data home    ]
	bind CombolaTCombobox <<CombolaGoEnd>>      [MkEgt <<CombolaGo>> -data end     ]
	bind CombolaTCombobox <<CombolaGoNextPage>> [MkEgt <<CombolaGo>> -data nextpage]
	bind CombolaTCombobox <<CombolaGoPrevPage>> [MkEgt <<CombolaGo>> -data prevpage]

	# Listbox
	bind CombolaTComboboxListbox <<CombolaListGoHome>>     [MkEgt <<CombolaListGo>> -data home    ]
	bind CombolaTComboboxListbox <<CombolaListGoEnd>>      [MkEgt <<CombolaListGo>> -data end     ]
	bind CombolaTComboboxListbox <<CombolaListGoNextPage>> [MkEgt <<CombolaListGo>> -data nextpage]
	bind CombolaTComboboxListbox <<CombolaListGoPrevPage>> [MkEgt <<CombolaListGo>> -data prevpage]

	# Scrollable
	bindAndBreak CombolaScrollableUpIsUp \
		<Alt-Key-Down>  continue                      \
		    <Key-Down>  [MkEgt <<CombolaGoNext>>    ] \
		    <Key-Up>    [MkEgt <<CombolaGoPrev>>    ] \
		    <Key-Next>  [MkEgt <<CombolaGoNextPage>>] \
		    <Key-Prior> [MkEgt <<CombolaGoPrevPage>>] \
	    <Control-Key-Home>  [MkEgt <<CombolaGoHome>>    ] \
	    <Control-Key-End>   [MkEgt <<CombolaGoEnd>>     ]

	bindAndBreak CombolaScrollableUpIsDown \
		<Alt-Key-Down>  continue                      \
		    <Key-Down>  [MkEgt <<CombolaGoPrev>>    ] \
		    <Key-Up>    [MkEgt <<CombolaGoNext>>    ] \
		    <Key-Next>  [MkEgt <<CombolaGoPrevPage>>] \
		    <Key-Prior> [MkEgt <<CombolaGoNextPage>>] \
	    <Control-Key-Home>  [MkEgt <<CombolaGoEnd>>     ] \
	    <Control-Key-End>   [MkEgt <<CombolaGoHome>>    ]

	# Combobox
	bindAndBreak CombolaTCombobox \
		<Alt-Key-Down>  [MkEgt <<CombolaOpenList>>  ] \
		<Alt-Key-space> [MkEgt <<CombolaOpenList>>  ]

#		<<TraverseIn>>  break

	# Listbox
	bind CombolaTComboboxListbox     <Key-Home>  [MkEgt <<CombolaListGoHome>>] 
	bind CombolaTComboboxListbox     <Key-End>   [MkEgt <<CombolaListGoEnd>>] 
	bind CombolaTComboboxListbox <Alt-Key-space> [MkEgt <Key-Escape>]
	bind CombolaTComboboxListbox     <Key-space> [MkEgt <Key-Escape>]

	bindAndBreak CombolaTComboboxListbox \
		    <Key-Next>  [MkEgt <<CombolaListGoNextPage>>] \
		    <Key-Prior> [MkEgt <<CombolaListGoPrevPage>>] \
		<Alt-Key-Up>	[MkEgt <Key-Escape>]

	# Key Select
	bind CombolaKeySelectTCombobox <Key>          [MkEgt <<CombolaKey>> -data %K]
	bind CombolaKeySelectTCombobox <<TraverseIn>> [MkEgt <<SelectAll>>]
	bind CombolaKeySelectTCombobox <FocusIn>      [MkEgt <<SelectAll>>]

	bindAndBreak CombolaKeySelectTCombobox \
		    <Key-Home>  [MkEgt <<CombolaGoHome>>  ] \
		    <Key-End>   [MkEgt <<CombolaGoEnd>>   ] \
		    <Key-Right> {} \
		    <Key-Left>  {} \
		    <Key-space> [MkEgt <<CombolaOpenList>>] \
		<Alt-Key-space> continue

	# Key Select Listbox
	bind CombolaKeySelectTComboboxListbox <Key> [MkEgt <<CombolaListKey>> -data %K]

	# Edit Ready
	bindAndBreak CombolaEditReadyTCombobox \
	    <Control-Key-Return> [MkEgt <<CombolaBeginEdit>>]

	# Edit
	bindAndBreak CombolaEditTCombobox \
	    <Control-Key-Return> [MkEgt <<CombolaAcceptEdit>> ] \
		    <Key-Escape> [MkEgt <<CombolaDiscardEdit>>]

	bind CombolaEditTCombobox <Button-1> {
		if {[%W identify element %x %y] in [list textarea downarrow]} {
			event generate %W <<CombolaOpenList>> -when tail
		}
		break
	}

if 0 {
	bindAndBreak CombolaEditTCombobox \
		<<NextWindow>>       {} \
		<<PrevWindow>>       {}
}

if 1 {
	set tns ::ttk::combobox:
if 1 {
	foreach {
		srcproc dstproc search replace
	} {
		PopdownWindow CombolaPopdownWindow
			{ComboboxListbox} {CombolaTComboboxListbox ComboboxListbox}
		Post CombolaPost
			{set popdown \[PopdownWindow \$cb\]} {set popdown [CombolaPopdownWindow $cb]}
		PopdownWindow CombolaKeySelectPopdownWindow
			{ComboboxListbox} {CombolaKeySelectTComboboxListbox CombolaTComboboxListbox ComboboxListbox}
		Post CombolaKeySelectPost
			{set popdown \[PopdownWindow \$cb\]} {set popdown [CombolaKeySelectPopdownWindow $cb]}
	} {
		if {[regsub $search [set body [info body $tns:$srcproc]] $replace body] != 1} {
			puts stderr "Couldn't substitute for $tns:$dstproc"
		}
		proc $tns:$dstproc {cb} $body
	}
} else {
	if {[regsub {ComboboxListbox} [info body $tns:PopdownWindow] {CombolaTComboboxListbox ComboboxListbox} body] != 1} {
		puts stderr "Couldn't create $tns:CombolaPopdownWindow"
	}
	proc $tns:CombolaPopdownWindow {cb} $body

	if {[regsub {set popdown \[PopdownWindow \$cb\]} [info body $tns:Post] {set popdown [CombolaPopdownWindow $cb]} body] != 1} {
		puts stderr "Couldn't create $tns:CombolaPost"
	}
	proc $tns:CombolaPost {cb} $body

	if {[regsub {ComboboxListbox} [info body $tns:PopdownWindow] {CombolaKeySelectTComboboxListbox CombolaTComboboxListbox ComboboxListbox} body] != 1} {
		puts stderr "Couldn't create $tns:CombolaKeySelectPopdownWindow"
	}
	proc $tns:CombolaKeySelectPopdownWindow {cb} $body

	if {[regsub {set popdown \[PopdownWindow \$cb\]} [info body $tns:Post] {set popdown [CombolaKeySelectPopdownWindow $cb]} body] != 1} {
		puts stderr "Couldn't create $tns:CombolaKeySelectPost"
	}
	proc $tns:CombolaKeySelectPost {cb} $body
}
} else {
	namespace eval ::ttk::combobox {

	proc CombolaPopdownWindow {cb} [string map {
		{ComboboxListbox} {CombolaTComboboxListbox ComboboxListbox}
	} [info body ::ttk::combobox::PopdownWindow]]

	proc CombolaPost {cb} [string map {
		{set popdown [PopdownWindow $cb]} {set popdown [CombolaPopdownWindow $cb]}
	} [info body ::ttk::combobox::Post]]

	proc CombolaKeySelectPopdownWindow {cb} [string map {
		{ComboboxListbox} {CombolaKeySelectTComboboxListbox CombolaTComboboxListbox ComboboxListbox}
	} [info body ::ttk::combobox::PopdownWindow]]

	proc CombolaKeySelectPost {cb} [string map {
		{set popdown [PopdownWindow $cb]} {set popdown [CombolaKeySelectPopdownWindow $cb]}
	} [info body ::ttk::combobox::Post]]
if 0 {
	proc CombolaPress {mode w x y} [string map {
		{Post $w} {CombolaPost $w}
	} [info body ::ttk::combobox::Press]]
}
	}; # namespace eval ::ttk::combobox
}

	set Initted yes
}


## Setup

variable Initted no

namespace import tu::tcl::getNextOrPrevMatch tu::tcl::whereBe
namespace import tu::tk::bindAndBreak

namespace ensemble create -command combola -prefixes no \
	-subcommands [list combobox enhance go] \
	-map [list go comboGo]

namespace export combola

package provide tu::combola 0.2


}}; # namespace tu::combola


# EOF
