#! /bin/sh
# \
exec wish${TCL} "$0" ${1+"$@"}

#
# Tu Epm Example
#
#
# Stuart Cassoff
#
# Fall 2022
# Version 1.0
#


package require Tcl 8.6


namespace eval tu::epm { namespace eval examples { namespace eval tu_epm_example {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu_epm_example
		title	{Epm Example}
		descr	{Using Edit Program Menu}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	1.0
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key not \"$key\"known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


### Tu Epm Example

proc go {args} {
	package require Tk
	package require tu::epm
	namespace import ::tu::epm::epmBindEpmMenuTo
	namespace import ::tu::epm::epmBindEditMenuTo
	namespace import ::tu::epm::epmBindMenuTo
	namespace import ::tu::epm::epmMenu

	set W . ; set w {}
	bind $W <Key-Escape> [list destroy $W]
	bind $W <Control-q>  [list destroy $W]

	wm title $W {Edit Program Menu}

	set ns [namespace current]:

	grid [set eFrame [ttk::labelframe $w.fE -text "Entry"]] -sticky ew -row 0 -column 0
	grid [set tFrame [ttk::labelframe $w.fT -text "Text"]] -sticky nsew -row 1 -column 0

	grid [set eEntry [ttk::entry $eFrame.eE]] -sticky ew
	grid [set tText [text $tFrame.tT -width 10 -height 5]] -sticky nsew

	foreach ww [list $W $eFrame $tFrame] {
		grid columnconfigure $ww 0 -weight 1 -uniform combos
	}
	foreach ww [list $W $tFrame] r [list 1 0] {
		grid rowconfigure $ww $r -weight 1
	}

	menu .m -tearoff 0
	.m add command -label abc
	.m add cascade -label qrz -menu [epmMenu]
	.m add command -label def

	epmBindEpmMenuTo $eEntry

#	epmBindEditMenuTo $tText
	epmBindMenuTo $tText .m


}

}}}; # namespace tu::epm::examples::tu_epm_example


if {[info exists ::argv0] && [info script] eq $::argv0} {
	tu::epm::examples::tu_epm_example::go {*}$::argv
}


# EOF
