#
# Tu MyFonts - Tcl Usefulness My Fonts
#
#
# Stuart Cassoff
#
# Summer 2022
# Version 0.1
#
# Fall
# Version 0.2
#

package require Tcl 8.5
package require TclOO

namespace eval tu { namespace eval myfonts {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu::myfonts
		title	{Tu MyFonts}
		descr	{My Fonts}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	0.2
	}]
	dict set pkgconfig importfrom [namespace parent]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


### Procedures copied from other places

namespace eval tu {
namespace eval tcl {
proc makeROAccessors {args} {
	set c ""
	foreach m $args { append c method " " $m " " "{}" " {" \
		[string trimright [regsub -all -line {^ +} [string map [list @m@ $m] {
			return $@m@
		}] "\t"]] "\n}\n"
	}
	return [string trimright $c]
}
}; # namespace eval tcl
}; # namespace eval tu


### MyFonts

::oo::class create MyFonts {
	variable TkFonts
	variable class
	variable fonts
	variable defaults
constructor {myClass args} {
	set class $myClass
	set TkFonts [list Default Text Fixed Menu Heading]
	set fonts [dict create]
	my setup {*}$args
}
eval [::tu::myfonts::tu::tcl::makeROAccessors class TkFonts fonts]
method setup {args} {
	foreach f $TkFonts {
		set c [font configure Tk${f}Font]
		dict set fonts [font create ${class}${f}Font {*}$c {*}$args] {}
	}
	foreach {attr which props} [list {*}{
		Bigger Text {-size 11}
		Log    Text {}
	}] {
		dict set fonts [font create ${class}${attr}${which}Font {*}[font configure ${class}${which}Font] {*}$props {*}$args] {}
	}
#		Bold   Text {-weight bold}
#		Bold Fixed   {-weight bold}
#		Big  Default {-size -18}

	foreach f [dict keys $fonts] { dict set fonts $f [font configure $f] }
	option add *${class}*Menu*font      ${class}MenuFont
	option add *${class}*Text*font      ${class}FixedFont
	option add *${class}*TLabel*font    ${class}DefaultFont
	option add *${class}*TEntry*font    ${class}TextFont
	option add *${class}*TCombobox*font ${class}TextFont
if 0 {
	ttk::style configure Heading           -font ${class}HeadingFont
	ttk::style configure Treeview          -font ${class}DefaultFont
	ttk::style configure TButton           -font ${class}DefaultFont
	ttk::style configure TCheckbutton      -font ${class}DefaultFont
	ttk::style configure TRadiobutton      -font ${class}DefaultFont
	ttk::style configure TNotebook.Tab     -font ${class}DefaultFont
	ttk::style configure TLabelframe.Label -font ${class}DefaultFont
}
}
method defaults {font} {
	return [dict get $fonts $font]
}
method reset {} {
	dict for {f c} $fonts { font configure $f {*}$c }
}
method transform-scale {{s 0} args} {
	if {[llength $args] > 0} {
		set fnts [dict create]
		foreach arg $args { dict set fnts $arg [dict get $fonts $arg] }
	} else {
		set fnts $fonts
	}
	dict for {f c} $fnts {
		set size [dict get $c -size]
		font configure $f -size [expr {$size + (($size < 0 ? -1 : 1) * $s)}]
	}
}
}; # ::oo::class create MyFonts


## Setup

#namespace export MyFonts

package provide tu::myfonts 0.2

}}; # namespace tu::myfonts


# EOF
