#
# Tu Pretty - Tcl Usefulness Pretty Printing
#
#
# Stuart Cassoff
#
# Summer 2022
# Version 0.1
#
# Fall 2022
# Version 0.2
#

package require Tcl 8.5

namespace eval tu { namespace eval pretty {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu::pretty
		title	{Tu Pretty}
		descr	{Pretty Printing}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	0.2
	}]
	dict set pkgconfig importfrom [namespace parent]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


# Tcl

proc hexdump {blob {bits 8}} {
	set l [string length $blob]
	set dump {}
	for {set i 0; set ii [expr {$bits - 1}]; set addr 0} {$i < $l} {incr i $bits; incr ii $bits; incr addr $bits} {
		binary scan [string range $blob $i $ii] @0H*@0a* hex ascii
		regsub -all -- {[^[:graph:] ]} $ascii {.} ascii
		regsub -all -- {..} $hex {& } hex
		lappend dump [format "%04X  %-[expr {3 * $bits}]s %-${bits}s" $addr $hex $ascii]
	}
	return $dump
}

proc prettyBindings {bindings} {
	set q ""
	dict for {k v} $bindings {
		append q $k \n
		dict for {b s} $v {
			append q "  " [format "%-30s" $b] " " $s \n
		}
	}
	return $q
}

proc mpd {dict {pat {*}}} {
	set f %-[::tcl::mathfunc::max 0 {*}[lmap k [set keys [lsort -dictionary [dict keys $dict $pat]]] { string length $k }]]s\ \ %s
	return [lmap k $keys { format $f $k [dict get $dict $k] }]
}
interp alias {} [namespace current]::makePrintDict {} [namespace current]::mpd

proc pd {dict {pat {*}}} {
	return [join [mpd $dict $pat] \n]
}
interp alias {} [namespace current]::printDict {} [namespace current]::pd

proc pa {arr {pat {*}}} {
	return [pd [array get $arr] $pat]
}
interp alias {} [namespace current]::printArray {} [namespace current]::pa

proc pv {var {pat {*}}} {
	upvar 1 $var v
	if {[array exists v]} { return [pa $var $pat] }
	if {![catch {dict size $v}]} { return [pd $v $pat] }
	return $v
}
interp alias {} [namespace current]::printVar {} [namespace current]::pv


## Tk
proc pb {tag} { package require tu::tk; return [prettyBindings [::tu::tk::bindings $tag]] }
interp alias {} prettyBindingsOfTag {} pb


## Setup

namespace export hexdump prettyBindings
namespace export mpd pv pa pd pb
namespace export makePrintDict printVar printArray printDict
namespace export prettyBindingsOfTag

package provide tu::pretty 0.2

}}; # namespace tu::pretty


# EOF
