#
# Tu Epm - Tcl Usefulness Tk Utils
#
#
# Stuart Cassoff
#
# Fall 2022
# Version 0.1
#

package require Tcl 8.5

namespace eval tu { namespace eval tk {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu::tk
		title	{Tu Tk}
		descr	{Tk Utils}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	0.1
	}]
	dict set pkgconfig importfrom [namespace parent]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


proc setEntryText {w text} {
	$w delete 0 end
	if {$text ne ""} { $w insert end $text }
}

proc setTextText {w text} {
	if {[set state [$w cget -state]] ne "normal"} { $w configure -state normal }
	$w delete 1.0 end
	if {$text ne ""} { $w insert end $text }
	#if {$text ne ""} { $w insert end $text[expr {[string index $text end] eq "\n" ? "" : "\n"}] }
	if {$state ne "normal"} { $w configure -state $state }
}

proc bindings {tag} {
	set d [dict create]
	set l [expr {[string index $tag 0] eq "." ? [bindtags $tag] : [list $tag]}]
	foreach t $l { foreach b [bind $t] {
		dict set d $t $b [bind $t $b]
	}}
	return $d
}

proc fb {window seq} {
	set found [list]
	foreach t [bindtags $window] {
		if {[set s [bind $t $seq]] eq ""} continue
		lappend found [list $t $s]
	}
	return $found
}
interp alias {} [namespace current]::findBinding {} [namespace current]::fb

proc cloneBinding {window fromseq toseq {map {}}} {
	set found [findBinding $window $fromseq]
	foreach tb $found {
		if {$map ne {}} {
			set tb [lmap e $tb { string map $map $e }]
		}
		bind {*}[linsert $tb 1 $toseq]
	}
	return $found
}

proc copyBindings {fromTag toTag} {
	foreach seq [bind $fromTag] {
		bind $toTag $seq [bind $fromTag $seq]
	}
}

# From the Tk widget demo "A multi-column list of countries"
# Code to do the sorting of the tree contents when clicked on
#
proc tvSortBy {tree col direction} {
	# Build something we can sort
	set data {}
	foreach row [$tree children {}] {
		lappend data [list [$tree set $row $col] $row]
	}

	set dir [expr {$direction ? "-decreasing" : "-increasing"}]
	set r -1

	# Now reshuffle the rows into the sorted order
	foreach info [lsort -dictionary -index 0 $dir $data] {
		$tree move [lindex $info 1] {} [incr r]
	}

	# Switch the heading so that it will sort in the opposite direction
	$tree heading $col -command [list [namespace current]::tvSortBy $tree $col [expr {!$direction}]]
}

#
# Underline Ampersand
# Convert ampersand in text to -underline option
# From Gub
#
proc ua {text} {	
	set index 0
	set uli -1
	while {[set index [string first "&" $text $index]] != -1} {
		if {[string index $text $index+1] eq "&"} {
			set text [string range $text 0 $index-1][string range $text $index+1 end]
			incr index
			continue
		}
		if {$index < [string length $text]-1} {
			set text [string range $text 0 $index-1][string range $text $index+1 end]
			set uli $index
		}
		incr index
	}
	set res [list $text]
	if {$uli != -1} {
		lappend res -underline $uli
	}
	return $res
}

proc bindAndBreak1 {w args} {
	foreach {seq script} $args {
		if {$script ni [list break continue]} {
			if {$script ne ""} { append script ";" }
			append script break
		}
		bind $w $seq $script
	}
}
proc bindAndBreak {w args} {
	dict for {seq script} $args {
		if {$script eq ""} {
			bind $w $seq break
		} elseif {$script in [list break continue]} {
			bind $w $seq $script
		} else {
			bind $w $seq $script
			bind $w $seq +break
		}
	}
}


## Setup

namespace export ua
namespace export bindings
namespace export fb
namespace export findBinding cloneBinding copyBindings
namespace export tvSortBy
namespace export setTextText setEntryText
namespace export bindAndBreak

package provide tu::tk 0.1

}}; # namespace tu::tk


# EOF
