#
# Tu Toplevels Tk - Tcl Usefulness Toplevels Tk
#
#
# Stuart Cassoff
#
# Summer 2022
# Version 0.1
#
# Fall 2022
# Version 0.2
#

package require Tcl 8.5

namespace eval tu { namespace eval toplevels { namespace eval tk {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	tu::toplevels::tk
		title	{Tu Toplevels Tk}
		descr	{Tk Toplevels}
		author	{Stuart Cassoff}
		when	{Fall 2022}
		version	0.2
	}]
	dict set pkgconfig importfrom [namespace parent]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


### Procedures copied from other places

namespace eval tu {
namespace eval tcl {
proc randstr {{length {6}}} {
	set str ""
	for {set i 0} {$i < $length} {incr i} {
		append str [format %c [expr {97 + int(rand() * 26)}]]
	}
	return $str
}
namespace export randstr
}; # namespace eval tcl
}; # namespace eval tu


### Tu Toplevels Tk

variable defaultWindow .

proc randWindow {{w {.}}} {
	while {[winfo exists [set rw $w[randstr]]]} {}
	return $rw
}

proc windowIsToplevel {w} {
#	return [expr {![string match .?*.?* $w]}]
#	return [expr {[winfo toplevel $w] eq $w}]
	return [expr {[winfo toplevel $w] eq $w || [winfo class $w] eq "Toplevel"}]
}

proc newToplevel {{w ""} args} {
	if {$w eq ""} { set w [randWindow] }
	toplevel $w {*}$args
	return $w
}

proc groupToplevel {w {W ""}} {
	variable defaultWindow
	if {$W eq ""} { set W $defaultWindow }
	wm group $w $W
}

proc destroyToplevel {w} {
	destroy $w
}

proc transientizeToplevel {w {W ""}} {
	variable defaultWindow
	if {$W eq ""} { set W $defaultWindow }
	wm transient $w $W
}

proc withdrawToplevel {w} {
	wm withdraw $w
}

proc forgetToplevel {w} {
	wm forget $w
}

proc iconifyToplevel {w} {
	wm iconify $w
}

proc deiconifyToplevel {w} {
	wm deiconify $w
}

proc toplevelIsZoomable {w} {
	return [expr {"-zoomed" in [dict keys [wm attributes $w]]}]
}

proc toplevelZoomed {w {zoomed {}}} {
	if {$zoomed ne {}} { wm attributes $w -zoomed $zoomed }
	return [wm attributes $w -zoomed]
}

proc toplevelFullscreen {w {fullscreen {}}} {
	if {$fullscreen ne {}} { wm attributes $w -fullscreen $fullscreen }
	return [wm attributes $w -fullscreen]
}

proc toplevelGeometry {w args} {
	if {[llength $args] == 1} { wm geometry $w [lindex $args 0] }
	return [wm geometry $w]
}

proc manageToplevel {w} {
	wm manage $w
}

proc toplevelResizable {w args} {
	if {[llength $args] == 2} { wm resizable $w {*}$args }
	return [wm resizable $w]
}

proc setToplevelTitle {w t} {
	wm title $w $t
}

proc getToplevelTitle {w} {
	return [wm title $w]
}

proc centreToplevel {w {W ""}} {
	variable defaultWindow
	if {$W eq ""} { set W $defaultWindow }
	::tk::PlaceWindow $w widget $W
}
interp alias {} [namespace current]::centerToplvel {} [namespace current]::centreToplvel

proc onToplevelDelete {w s} {
	if {[winfo toplevel $w] eq $w} {
		wm protocol $w WM_DELETE_WINDOW $s
	} else {
		bind $w <Destroy> $s
	}
}

proc createToplevelGui {{title ""} args} {
	set w [newToplevel {} {*}$args]
	groupToplevel $w
	if {$title ne ""} { setToplevelTitle $w $title }
	return $w
}


## Setup

namespace import tu::tcl::randstr

namespace export \
	randWindow windowIsToplevel newToplevel groupToplevel destroyToplevel \
	transientizeToplevel centreToplevel centerToplevel \
	withdrawToplevel forgetToplevel iconifyToplevel deiconifyToplevel \
	onToplevelDelete setToplevelTitle getToplevelTitle \
	toplevelIsZoomable toplevelZoomed toplevelFullscreen \
	toplevelGeometry manageToplevel toplevelResizable \
	createToplevelGui

package provide tu::toplevels::tk 0.2

}}}; # namespace tu::toplevels::tk


# EOF
